# Python files compilation macro
# It processes a list of python files, copies them to the build dir and compiles them
# It returns the list of .pyc filenames
#
# Panagiotis Skintzos 03/2011
#

macro(python_compile output)
    set(${output} "")
    set(_source_files ${ARGV})
    set(_dest_files "")
    set(_cleaned_files "")

    list(REMOVE_AT _source_files 0)
    foreach(file ${_source_files})
        if(NOT (${CMAKE_CURRENT_SOURCE_DIR} STREQUAL ${CMAKE_CURRENT_BINARY_DIR}))
            list(APPEND _cleaned_files "${file}")
            configure_file( ${file} ${file} COPYONLY )
        endif(NOT (${CMAKE_CURRENT_SOURCE_DIR} STREQUAL ${CMAKE_CURRENT_BINARY_DIR}))

        list(APPEND _dest_files "${CMAKE_CURRENT_BINARY_DIR}/${file}c")
        add_custom_command(
            OUTPUT "${CMAKE_CURRENT_BINARY_DIR}/${file}c"
            DEPENDS ${CMAKE_CURRENT_BINARY_DIR}/${file}
            COMMAND ${PYTHON_EXECUTABLE} -m compileall -f ${file}
        )
    endforeach(file ${_source_files})

    get_directory_property(_clean_files ADDITIONAL_MAKE_CLEAN_FILES)
    list(APPEND _clean_files "${_cleaned_files}")
    set_directory_properties(PROPERTIES ADDITIONAL_MAKE_CLEAN_FILES "${_clean_files}")

    set(${output} ${_dest_files})
endmacro(python_compile)

macro(python_compile_no_copy output)
    set(${output} "")
    set(_source_files ${ARGV})
    set(_dest_files "")
    set(_cleaned_files "")

    list(REMOVE_AT _source_files 0)
    foreach(file ${_source_files})
        list(APPEND _dest_files "${CMAKE_CURRENT_BINARY_DIR}/${file}c")
        add_custom_command(
            OUTPUT "${CMAKE_CURRENT_BINARY_DIR}/${file}c"
            DEPENDS ${CMAKE_CURRENT_BINARY_DIR}/${file}
            COMMAND ${PYTHON_EXECUTABLE} -m compileall -f ${file}
        )
    endforeach(file ${_source_files})

    get_directory_property(_clean_files ADDITIONAL_MAKE_CLEAN_FILES)
    list(APPEND _clean_files "${_cleaned_files}")
    set_directory_properties(PROPERTIES ADDITIONAL_MAKE_CLEAN_FILES "${_clean_files}")

    set(${output} ${_dest_files})
endmacro(python_compile_no_copy)
