# add custom target distclean
# cleans and removes cmake generated files etc.
#
# Jan Woetzel 04/2003
# Panagiotis Skintzos 03/2011
#

macro(distclean_recursive output)
    set(${output} "")

    set(_distclean_subdirs "${CMAKE_CURRENT_BINARY_DIR}")
    SET(_distclean_filenames
            cmake.depends
            cmake.check_depends
            CMakeCache.txt
            cmake.check_cache
            *.cmake
            Makefile
            install_manifest.txt
            CMakeFiles
    )

    set(_subdirs_filename "${CMAKE_CURRENT_BINARY_DIR}/CMakeFiles/TargetDirectories.txt")
    if(EXISTS ${_subdirs_filename})
        file(STRINGS ${_subdirs_filename} _target_subdirs)
        foreach(dirpath ${_target_subdirs})
            string(REGEX REPLACE "/CMakeFiles/.*$" "" dirpath ${dirpath})
            list(APPEND _distclean_subdirs "${dirpath}")
        endforeach(dirpath ${_target_subdirs})
    endif(EXISTS ${_subdirs_filename})
    list(SORT _distclean_subdirs)
    list(REMOVE_DUPLICATES _distclean_subdirs)
    list(REVERSE _distclean_subdirs)

    set(_distclean_paths "")
    foreach(dirpath ${_distclean_subdirs})
        #execute_process(COMMAND echo "path to clean: ${dirpath}")
        foreach(filename ${_distclean_filenames})
            list(APPEND _distclean_paths "${dirpath}/${filename}")
            #execute_process(COMMAND echo "file to clean:  ${dirpath}/${filename}")
        endforeach(filename ${_distclean_filenames})
    endforeach(dirpath ${_distclean_subdirs})

    set(${output} ${_distclean_paths})

endmacro(distclean_recursive)

IF (UNIX)
  ADD_CUSTOM_TARGET (distclean @echo cleaning for source distribution)
  distclean_recursive(DISTCLEANED)

  ADD_CUSTOM_COMMAND(
    DEPENDS clean
    COMMENT "distribution clean"
    COMMAND rm
    ARGS    -rf ${DISTCLEANED}
    TARGET  distclean
  )

ENDIF(UNIX)
