#!/usr/bin/perl
# Reads a C file, collects the constants and writes python code
#
# Panagiotis Skintzos 03/2011
#
$pattern=shift;
$filename=shift;
# read whole file
open($file,"<",$filename) or die "Can't open ${filename}: $!";
$contents = do { local $/; <$file> };
# join multiline escaped strings
$contents =~ s/\"\s*\\\n\"//;
# we filter string or numeric constants
use constant BEGINNING => '^(#define)?\s+';         #start with space or #define
use constant NAME => '(?<name>\w+)\s*';             #alphanumeric
use constant SEPARATOR => '(=|\(\(guint\))?\s*';    #space or equal or guint
use constant VALUE => '(?<value>(\".*\"|[\d-.]+))';     #literal string or number
use constant ENDING => '[\s,)]*$';        #end with space, comma, or parenthesis
# build regex
$regex = qr/${\(BEGINNING)}${pattern}${\(NAME)}${\(SEPARATOR)}${\(VALUE)}${\(ENDING)}/;
# split to lines
@lines = split('\n', $contents);
foreach $line (@lines) {
    print "$+{name} = $+{value}\n" if ($line =~ $regex);
}
