/* menuitem_separator.vala
 *
 * Copyright(C) 2011  Panagiotis Skintzos
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 *(at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 *     Panagiotis Skintzos <panajotis@gmail.com>
 */

using Gtk;

namespace IndicatorWeather {

/**
 * Separator menuitem
 *
 * Subclass of Gtk.SeparatorMenuItem
 *
 * A simple separator, which listens to our service interface
 *
 **/
class SeparatorMenuItem : Gtk.SeparatorMenuItem {

    /**
     * Reference to the Dbus menuitem we listen to.
     * Defined as weak to avoid cycled refs.
     */
    private weak Dbusmenu.Menuitem _dbusitem = null;

    /**
     * The menuitem key used for debugging
     */
    private string _key;

    /**
     * Reference to the service interface.
     */
    private ServiceClient _service_client = null;

    /**
     * Hidden constructor
     */
    private SeparatorMenuItem(Dbusmenu.Menuitem item) {

        _dbusitem = item;
        _key = _dbusitem.property_get(MENUITEM_PROP_KEY);

        name = "%s[%s]".printf(this.get_type().name().
            replace("IndicatorWeather", "").replace("MenuItem", ""), _key);
        debug("%s created", name);

        try {
            _service_client = Bus.get_proxy_sync(BusType.SESSION,
                DBUS_SERVICE_NAME2, DBUS_SERVICE_OBJ,
                DBusProxyFlags.NONE, null);
        } catch (Error e) {
            critical("%s init: service error: %s", name, e.message);
        }

        _update();
    }

    /**
     * Standard construction method called from the dbusmenu client.
     *
     * Creates an instance and registers it.
     */
    public static bool new_menuitem (Dbusmenu.Menuitem newitem,
        Dbusmenu.Menuitem parent, Dbusmenu.Client client) {

        ((DbusmenuGtk.Client)client).newitem_base(newitem,
            new SeparatorMenuItem(newitem), parent);
        return true;
    }

    public override void destroy() {
        debug("%s destroy", name);
        base.destroy();
    }

    public override void realize() {
        debug("%s realized", name);
        base.realize();
    }

    private void _update() {
        // update the widget
        if (_dbusitem.property_exist(Dbusmenu.MENUITEM_PROP_VISIBLE))
            set_visible(_dbusitem.property_get_bool(
                Dbusmenu.MENUITEM_PROP_VISIBLE));

        // setup the property_changed listener
        if (USE_SERVICE_CLIENT_PROP_CHANGE && _service_client != null)
            _service_client.menuitem_property_changed.connect(
                on_menuitem_property_changed);
        else
            _dbusitem.property_changed.connect(on_property_changed);
    }

    public void on_menuitem_property_changed (string menuitem_key,
            string property, GLib.Variant value) {
        if (menuitem_key == _key)
            on_property_changed(property, value);
    }

    public void on_property_changed (string property, GLib.Variant value) {
        if (property == Dbusmenu.MENUITEM_PROP_VISIBLE) {
            set_visible(value.get_boolean());

        } else {
            debug("%s: unhandled property changed: %s", name, property);
        }
    }

}

//namespace IndicatorWeather
}

