#!/usr/bin/env python
# -*- coding: utf-8 -*-
### BEGIN LICENSE
# Copyright (C) 2010 Sebastian MacDonald Sebas310@gmail.com
# Copyright (C) 2010 Mehdi Rejraji mehd36@gmail.com
# Copyright (C) 2011 Vadim Rutkovsky roignac@gmail.com
# Copyright (C) 2011 Panagiotis Skintzos panajotis@gmail.com
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import sys
import os
import logging, logging.handlers
import optparse

from indicator_weather_service import Server

if __name__ == '__main__':

    parser = optparse.OptionParser(
        usage="usage: %prog [options]",
        version="%prog 1.0",
        description="Indicator-Weather dbus service"
    )
    parser.add_option("-k", "--keep-alive",
                  action="store_true", dest="keepalive", default=False,
                  help="keep the service alive, even if no client is connected")
    parser.add_option("-v", "--verbose",
                  action="store_true", dest="verbose", default=False,
                  help="print info messages")
    parser.add_option("-d", "--debug",
                  action="store_true", dest="debug", default=False,
                  help="print debug messages")

    (options, args) = parser.parse_args()

    #Enable and configure logs
    log = logging.getLogger('IndicatorWeatherService')
    log.propagate = False
    log.setLevel(logging.DEBUG)
    cachedir = os.environ.get('XDG_CACHE_HOME','').strip()
    if not cachedir:
        cachedir = "~/.cache")
    log_filename = os.path.join(os.path.expanduser(cachedir),
                                "indicator-weather-service.log")
    log_handler = logging.handlers.RotatingFileHandler(log_filename,
        maxBytes=1024*1024*10, backupCount=5)
    log_formatter = logging.Formatter(
        "[%(threadName)s] %(asctime)s - %(levelname)s - %(message)s")
    log_handler.setFormatter(log_formatter)
    log_handler.setLevel(logging.DEBUG)
    log.addHandler(log_handler)

    console_handler = logging.StreamHandler()
    console_formatter = logging.Formatter(
        "[%(threadName)s] %(levelname)s - %(message)s")
    console_handler.setFormatter(console_formatter)
    if options.debug:
        console_handler.setLevel(logging.DEBUG)
    elif options.verbose:
        console_handler.setLevel(logging.INFO)
    else:
        console_handler.setLevel(logging.WARNING)
    log.addHandler(console_handler)

    rootdir = os.path.abspath(os.path.dirname(os.path.realpath(sys.argv[0])))
    log.info("------------------------------")
    log.info("Started Weather Indicator Service from %s" % rootdir)

    server=Server(log, options.keepalive)
    server.start()

