#!/usr/bin/env python
# -*- coding: utf-8 -*-
### BEGIN LICENSE
# Copyright (C) 2010 Sebastian MacDonald Sebas310@gmail.com
# Copyright (C) 2010 Mehdi Rejraji mehd36@gmail.com
# Copyright (C) 2011 Vadim Rutkovsky roignac@gmail.com
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE


__all__ = [
    'Location',
    ]

import re
import urllib2
from urllib import urlencode
from xml.dom.minidom import parseString

from indicator_weather_service import *
from .weather import WeatherSource

class Location(object):
    """ Data object to store location details """

    def __init__(self, location_details, log):
        """
        Initialize
        """
        self.location_details = location_details
        self.log = log

    def prepare_location(self, geonames_details):
        """
        Gets necessary location details by its GeoNames details
        """
        self.location_details = {}
        self.location_details['full name'] = geonames_details[0]
        self.location_details['latitude'] = geonames_details[2]
        self.location_details['longitude'] = geonames_details[3]

        # get a code
        code = self.get_location_code(geonames_details)
        if code is None:
            return False
        self.location_code = code
        self.log.debug("Location: code for '%s': '%s'" % \
            (self.location_details['full name'], self.location_code))

        # call provider classes to get weather source specific id
        found_one = False
        for key in WeatherSource.INFO:
            if INFO_LABEL in WeatherSource.INFO[key]:
                label = WeatherSource.INFO[key][INFO_LABEL]
                cls = WeatherSource.load_class(key)
                loc_id = cls.get_location_id(self.location_code,
                    self.location_details, self.log)
                self.log.debug("Location: '%s' for '%s': '%s'" % \
                    (label, self.location_code, loc_id))
                if loc_id is not None:
                    self.location_details[label] = loc_id
                    found_one = True

        # check that we have at least one supported data source
        if not found_one:
            self.log.error(("Location '%s'" %
                self.location_details['full name']) + \
                "is not supported by current data sources")
            return False

        return True

    def get_location_code(self, geonames_details):
        """
        Returns a unique code for a location
        """
        # Get location details in english for Yahoo
        baseurl = 'http://api.geonames.org/getJSON'
        params = { 'geonameId': geonames_details[1],
                   'username': 'indicatorweather' }
        url = '?'.join((baseurl, urlencode(params)))
        self.log.debug("Location: Get GeoNames location details, url %s" % url)
        try:
            city = eval(urllib2.urlopen(url).read())
            if 'adminName1' in city:
                displayed_city_name = "%s, %s, %s" % \
                    (city['name'], city['adminName1'], city['countryName'])
            elif 'name' in city:
                displayed_city_name = "%s, %s" % \
                    (city['name'], city['countryName'])
            else:
                self.log.error("Location: Cannot find GeoNames info for " +
                    "code %s Full Response:\n %s" % (
                        geonames_details[1], str(city)))
                return

            # Get YAHOO WOEID by english name of location
            baseurl = 'http://where.yahooapis.com/geocode'
            params = {'location': displayed_city_name,
                      'appid': 'mOawLd4s', 'flags': 'J' }
            url = '?'.join((baseurl, urlencode(params)))
            self.log.debug("Location: Get Yahoo WOEID, url %s" % url)
            f = urllib2.urlopen(url)
            s=f.read()
            null = None
            yahoo_woeid_result = eval(s)
            if (yahoo_woeid_result['ResultSet']['Error'] != 0) and \
                    (yahoo_woeid_result['ResultSet']['Results'] != None):
                self.log.error("Location: Yahoo woeid return error. " +
                    "Full response:\n %s" % str(yahoo_woeid_result))
            else:
                return yahoo_woeid_result['ResultSet']['Results'][0]['woeid']

        except urllib2.URLError:
            self.log.error("Location: error reaching url '%s'" % url)

        except Exception, e:
            self.log.error(e)
            self.log.debug(traceback.format_exc(e))
        return None

    def export_location_details(self):
        """
        Return location code and location details
        """
        return (self.location_code, self.location_details)

