#!/usr/bin/env python
# -*- coding: utf-8 -*-
### BEGIN LICENSE
# Copyright (C) 2010 Sebastian MacDonald Sebas310@gmail.com
# Copyright (C) 2010 Mehdi Rejraji mehd36@gmail.com
# Copyright (C) 2011 Vadim Rutkovsky roignac@gmail.com
# Copyright (C) 2011 Panagiotis Skintzos panajotis@gmail.com
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import traceback
import datetime

from indicator_weather_service import *

class WeatherProvider(object):
    """
    """
    TIMESTAMP_FORMAT = "%Y%m%d%H%M"
    NAME = "DEFINE_ME"

    def __init__(self, location_id, lat, lon, log):
        """
        Initialize
        """
        self.log = log
        self._location_id = location_id
        self._lat = lat
        self._lon = lon

    def fetch_data(self):
        """
        This should be implemented by subclasses
        and return a dict with raw data values.
        The dict keys should be taken from WeatherSection class:
        {
        'info' = { 'field' = value, ...},
        'current' = { 'field' = value, ...},
        'forecast0' = { 'field' = value, ...},
        ...
        'forecastN' = { 'field' = value, ...},
        }
        """
        return None

    # ------------------------------------------------------------
    # Helpers
    # ------------------------------------------------------------

    @staticmethod
    def _get_timestamp():
        return datetime.datetime.now()

    def _get_timestamp_from_string(self, s):
        try:
            return datetime.datetime.strptime(s,
                WeatherProvider.TIMESTAMP_FORMAT)
        except Exception, e:
            self.log.error(e)
            self.log.debug(traceback.format_exc(e))
        return self._get_timestamp()

    def _get_string_from_timestamp(self, t):
        try:
            return t.strftime(WeatherProvider.TIMESTAMP_FORMAT)
        except Exception, e:
            self.log.error(e)
            self.log.debug(traceback.format_exc(e))
        return self._get_timestamp().strftime(
            WeatherProvider.TIMESTAMP_FORMAT)

    def _extract_str(self, section, key):
        try:
            return section[key]
        except Exception, e:
            self.log.error("WeatherProvider[%s]: parsing %s: %s\n%s" % \
                (self.NAME, key, e, section))
        return None

    def _extract_float(self, section, key):
        try:
            return float(section[key])
        except Exception, e:
            self.log.error("WeatherProvider[%s]: parsing %s: %s\n%s" % \
                (self.NAME, key, e, section))
        return None

    def _extract_int(self, section, key):
        try:
            return int(section[key])
        except Exception, e:
            self.log.error("WeatherProvider[%s]: parsing %s: %s\n%s" % \
                (self.NAME, key, e, section))
        return None

    def _extract_time(self, section, key, format):
        try:
            return datetime.datetime.strptime(section[key], format).time()
        except Exception, e:
            self.log.error("WeatherProvider[%s]: parsing %s: %s\n%s" % \
                (self.NAME, key, e, section))
        return None

    def _extract_date(self, section, key, format):
        try:
            return datetime.datetime.strptime(section[key], format).date()
        except Exception, e:
            self.log.error("WeatherProvider[%s]: parsing %s: %s\n%s" % \
                (self.NAME, key, e, section))
        return None
