#!/usr/bin/env python
# -*- coding: utf-8 -*-
### BEGIN LICENSE
# Copyright (C) 2010 Sebastian MacDonald Sebas310@gmail.com
# Copyright (C) 2010 Mehdi Rejraji mehd36@gmail.com
# Copyright (C) 2011 Vadim Rutkovsky roignac@gmail.com
# Copyright (C) 2011 Panagiotis Skintzos panajotis@gmail.com
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE


__all__ = [
    'CachedWeather',
    ]

import types
import traceback

from indicator_weather_service import *
from indicator_weather_service.providers import WeatherProvider
from indicator_weather_service.weather import WeatherField, WeatherSection

class CachedWeather(WeatherProvider):
    """
    WeatherProvider subclass for cached weather data
    """
    NAME = "Cache"

    def __init__(self, location_code, settings, log):
        """
        Initialize
        """
        super(CachedWeather, self).__init__(location_code,
            None, None, log)
        try:
            self.__report = eval(settings.get_weather(location_code))
            if not isinstance(self.__report, types.DictType):
                self.__report = None
        except Exception, e:
            self.log.warning("CachedWeather: %s" % e)
            self.__report = None

    def fetch_data(self):
        """
        Gets weather data from cache
        """
        self.log.debug("CachedWeather: report for %s:\n%s" % \
            (self._location_id, self.__report))
        # parse data
        return self.__parse_data()

    def __parse_data(self):
        """
        Parses the received data and fills in a dict
        """
        if self.__report is None:
            return None

        data = {}
        # grab the sections
        for key in WeatherSection.ALL_SECTIONS:
            if key in self.__report:
                data[key] = self.__report[key]

        # validation and type conversion
        for key in data:
            section = data[key]
            if not isinstance(section, types.DictType):
                data[key] = {}

            for field in section:
                try:
                    field_type = WeatherField.INFO[field].get(INFO_TYPE)
                    if field_type == types.StringType:
                        pass

                    elif field_type == datetime.datetime:
                        data[key][field] = self._get_timestamp_from_string(
                            section[field])

                    elif field_type == types.FloatType:
                        data[key][field] = float(section[field])

                    elif field_type == types.IntType:
                        data[key][field] = int(section[field])

                    elif field_type == types.BooleanType:
                        data[key][field] = True \
                            if section[field]=="True" else False

                    else:
                        raise Exception("Unhandled type: %s" % field_type)

                except Exception, e:
                    self.log.warning(e)
                    data[key].pop(field)

        return data
