#!/usr/bin/env python
# -*- coding: utf-8 -*-
### BEGIN LICENSE
# Copyright (C) 2010 Sebastian MacDonald Sebas310@gmail.com
# Copyright (C) 2010 Mehdi Rejraji mehd36@gmail.com
# Copyright (C) 2011 Vadim Rutkovsky roignac@gmail.com
# Copyright (C) 2011 Panagiotis Skintzos panajotis@gmail.com
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

__all__ = [
    'ServiceInterface',
    ]

from indicator_weather_service import *

import dbus
import dbus.service

class ServiceInterface(dbus.service.Object):

    def __init__(self, bus, indicator):
        self.indicator = indicator

        # register a name for the service
        self.indicator.log.debug("ServiceInterface: " + \
            "Registering DBus Service: '%s'" % DBUS_SERVICE_NAME2)
        answer = bus.request_name(DBUS_SERVICE_NAME2,
                                  dbus.bus.NAME_FLAG_DO_NOT_QUEUE)
        if answer != dbus.bus.REQUEST_NAME_REPLY_PRIMARY_OWNER:
            reply=answer
            if answer == dbus.bus.REQUEST_NAME_REPLY_ALREADY_OWNER:
                reply="name is already owned by us"
            elif answer == dbus.bus.REQUEST_NAME_REPLY_IN_QUEUE:
                reply="name is in queue"
            elif answer == dbus.bus.REQUEST_NAME_REPLY_EXISTS:
                reply="name already exists"
            raise Exception("Failed acquiring the service name " +
                "'%s': %s" % (DBUS_SERVICE_NAME2, reply))

        # initialize the interface
        dbus.service.Object.__init__(self,
            dbus.service.BusName(DBUS_SERVICE_NAME2, bus,
                do_not_queue=True), DBUS_SERVICE_OBJ)

    @dbus.service.signal(dbus_interface=DBUS_SERVICE_IFACE,
                        signature='i')
    def DisplayModeChanged(self, display_mode):
        pass

    @dbus.service.signal(dbus_interface=DBUS_SERVICE_IFACE,
                        signature='s')
    def IconNameChanged(self, icon_name):
        pass

    @dbus.service.signal(dbus_interface=DBUS_SERVICE_IFACE,
                        signature='s')
    def LabelMarkupChanged(self, label_markup):
        pass

    @dbus.service.signal(dbus_interface=DBUS_SERVICE_IFACE,
                        signature='ssv')
    def MenuitemPropertyChanged(self, key, property, value):
        pass

    @dbus.service.method(dbus_interface=DBUS_SERVICE_IFACE,
                        in_signature='', out_signature='i')
    def GetDisplayMode(self):
        return self.indicator.settings.indicator_display

    @dbus.service.method(dbus_interface=DBUS_SERVICE_IFACE,
                        in_signature='', out_signature='s')
    def GetIconName(self):
        return self.indicator.indicator_icon

    @dbus.service.method(dbus_interface=DBUS_SERVICE_IFACE,
                        in_signature='', out_signature='s')
    def GetLabelMarkup(self):
        return self.indicator.indicator_label

    @dbus.service.method(dbus_interface=DBUS_SERVICE_IFACE,
                        in_signature='s', out_signature='')
    def SetLocation(self, key):
        self.indicator.set_location(key)
